/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *
 *  File:	nf_tqt.h
 *  Purpose:	Some tqt-specific definitions and functions
 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef __NF_TQT_H_
#define __NF_TQT_H_

#include <linux/netdevice.h>

/* Structure to store the run-time configurable debug switches.
 * It controls which information is to put in the system log. See the output of 'dmesg'. */
typedef struct {
    int
        /* If it is on (1), some basic events on packets are printed during conntrack processing. */
        packet_path:1,
        /* This flag extends the 'packet_path': more deatils are printed. */
        packet_verbose:1,
        /* If this is set, the basic conntrack manipulations are printed. */
        conntrack:1,
        /* If it is on, the tqt-specific conntrack table gets more verbose. */
        conntrack_verbose:1,
        /* If it is on, the operations within procfs are printed. */
        procfs:1;
} tqt_debuglevel_type;

#define TQT_DEBUGLEVEL_STRINGS  { \
        "Debug for packet path", \
        "Verbose packet debug", \
        "Debug for conntrack manipulations (create, destroy, etc)", \
        "Verbose conntrack table", \
        "Proc fs debug" \
}

extern tqt_debuglevel_type tqt_debuglevel;

#define TQT_DEBUG(mask, info)   { if (tqt_debuglevel.mask) printk(info); }

static inline const char* get_dev_name_p(struct net_device* dev)
{
    return dev ? dev->name : "?";
}

static const char* get_dev_name(int dev_id)
{
    if (dev_id > 0) {
        struct net_device* dev = dev_get_by_index(&init_net, dev_id);
        if (dev) {
            dev_put(dev);
            return dev->name;
        }
    }
    return "?";
}

#define __get_tuple_address_mode(t)	\
	(t).followaddress ? ",fa" :	\
	(t).ignoreaddress  ? ",ia" : ""

#define __get_tuple_port_mode(t)	\
	(t).followport ? ",fp" :	\
	(t).ignoreport ? ",ip" : ""

#define print_tuple_dbg(tuple) \
{ \
    printk("%s:%u.%u.%u.%u%s%s:%d(%x)->%s:%u.%u.%u.%u:%d(%x)", \
           get_dev_name((tuple)->src.dev), NIPQUAD((tuple)->src.u3.ip), \
           __get_tuple_address_mode((tuple)->src), __get_tuple_port_mode((tuple)->src), \
           (int)ntohs((tuple)->src.u.udp.port), (tuple)->src.tos & 0xff, \
           get_dev_name((tuple)->dst.dev), \
           NIPQUAD((tuple)->dst.u3.ip), (int)ntohs((tuple)->dst.u.udp.port), (tuple)->dst.tos & 0xff); \
}

/* The TSO has 8-bit size, and a flag must be stored to indicate if the value is set or not. It is stored on 32 bit
 * here: the lower 8 bits contain the value, and this bit is a flag.
 * */
#define TQT_TOS_FLAG	0x10000

/* This macro reads one bit from a packed array: a char[] array can be accessed bit-by-bit this way.
 * */
#define GET_PORT_FLAG(p, f) (p[f>>3] &  (1<<(f&7)))

/* This macro sets one bit in a packed array: a char[] array can be accessed bit-by-bit this way.
 * */
#define SET_PORT_FLAG(p, f)  p[f>>3] |= (1<<(f&7))

#endif // __NF_TQT_H_

/* - - - - - - - - - - - - - - End - of - File - - - - - - - - - - - - - - */
